use DataScience
go

-- create and populate a fresh copy of SPYvsPYPL
-- for period from 2020-04-01 through 2021-13-15
drop table if exists SPYvsSOXL

create table SPYvsSOXL
(
start_date date not null,
end_date date not null,
SPY_Close_percent_gain real not null,
SOXL_Close_percent_gain real not null
)
go

insert into SPYvsSOXL
values
	('4-1-2020', '4-30-2020',   0.127,   0.3828),
	('5-1-2020','5-31-2020',    0.0476,  0.1763),
	('6-1-2020','6-30-2020',    0.0133,  0.2212),
	('7-1-2020','7-31-2020',    0.0589,  0.2036),
	('8-1-2020','8-31-2020',    0.0698,  0.1792),
	('9-1-2020','9-30-2020',   -0.0413, -0.0521),
	('10-1-2020','10-31-2020', -0.0249, -0.0167),
	('11-1-2020','11-30-2020',  0.1088,  0.6307),
	('12-1-2020','12-31-2020',  0.0326,  0.149 ),
	('1-1-2021','1-31-2021',   -0.0102,  0.074 ),
	('2-1-2021','2-28-2021',    0.0278,	 0.159 ),
	('3-1-2021','3-15-2021',    0.0409, -0.0447)




-- create and populate a fresh copy of SPYvsPYPL
-- for period from 2020-04-01 through 2021-13-15
drop table if exists SPYvsPYPL

create table SPYvsPYPL
(
start_date date not null,
end_date date not null,
SPY_Close_percent_gain real not null,
PYPL_Close_percent_gain real not null
)

insert into SPYvsPYPL
values
	('4-1-2020', '4-30-2020',   0.127,   0.2847),
	('5-1-2020','5-31-2020',    0.0476,  0.2602),
	('6-1-2020','6-30-2020',    0.0133,  0.124 ),
	('7-1-2020','7-31-2020',    0.0589,  0.1254),
	('8-1-2020','8-31-2020',    0.0698,  0.0412),
	('9-1-2020','9-30-2020',   -0.0413, -0.0348),
	('10-1-2020','10-31-2020', -0.0249, -0.0553),
	('11-1-2020','11-30-2020',  0.1088,  0.1504),
	('12-1-2020','12-31-2020',  0.0326,  0.0938),
	('1-1-2021','1-31-2021',   -0.0102,  0.0005),
	('2-1-2021','2-28-2021',    0.0278,	 0.109 ),
	('3-1-2021','3-15-2021',    0.0409, -0.0406)



-- SPYvsPYPL and SPYvsPYPL SUCCESS_OR_FAIL results
-- Answers: does symbol rise more than SPY benchmark?
select 
	*
   ,case
		WHEN SOXL_Close_percent_gain > SPY_Close_percent_gain then 'SUCCESS'
		ELSE 'FAIL'
    end SUCCESS_OR_FAIL
from SPYvsSOXL

select 
	*
   ,case
		WHEN PYPL_Close_percent_gain > SPY_Close_percent_gain then 'SUCCESS'
		ELSE 'FAIL'
    end SUCCESS_OR_FAIL
	from SPYvsPYPL


-- create a table for storing values used to compute
-- the statistical significance of the outcomes in a sign test dataset
drop table if exists Binomial_probability_calculator_for_sign_test

create table Binomial_probability_calculator_for_sign_test
(
symbol varchar(10) not null,
s int not null,
t int not null,
t_less_s int not null,
s_factorial bigint not null,
t_factorial bigint not null,
t_less_s_factorial bigint not null,
p float default .5,
binomial_coeff int,
p_to_s_power float,
p_to_t_less_s_power float,
sPt float
)



-- compuute and insert inputs for computing binomial proability 
-- of s successes in t trials where the probabability of 
-- of success and failure per trial equals .5

insert Binomial_probability_calculator_for_sign_test
(
symbol,
s,
t,
t_less_s,
s_factorial,
t_factorial,
t_less_s_factorial
)

(
-- aggregate successes (s), trials (t), and t_less_s with factorials for SOXL
select 
	symbol
   ,s
   ,t
   ,t_less_s
   ,(select dbo.factorial(s)) s_factorial
   ,(select dbo.factorial(t)) t_factorial
   ,(select dbo.factorial(t_less_s)) t_less_s_factorial
from
(
-- cross joined aggregate success and trial data with trials less successes --factorials for SOXL
select 
	symbol symbol
   ,successes s
   ,trials t
   ,(trials - successes) t_less_s
from (
select 'SOXL' 'SYMBOL', count(*) Successes
from
(
-- trial data
select 
	*
   ,case
		WHEN SOXL_Close_percent_gain > SPY_Close_percent_gain then 'SUCCESS'
		ELSE 'FAIL'
    end SUCCESS_OR_FAIL
from SPYvsSOXL
) for_SUCCESS_COUNT_for_SOXL
group by SUCCESS_OR_FAIL
having SUCCESS_OR_FAIL = 'SUCCESS') for_aggregate_successes
cross join
(select count(*) trials from SPYvsSOXL) for_aggregate_trials
) for_aggregate_trials_with_factorials

union


-- aggregate successes (s), trials (t), and t_less_s with factorials for PYPL
select 
	symbol
   ,s
   ,t
   ,t_less_s
   ,(select dbo.factorial(s)) s_factorial
   ,(select dbo.factorial(t)) t_factorial
   ,(select dbo.factorial(t_less_s)) t_less_s_factorial
from
(
-- cross joined aggregate success and trial data with trials less successes --factorials for PYPL
select 
	symbol symbol
   ,successes s
   ,trials t
   ,(trials - successes) t_less_s
from (
select 'PYPL' 'SYMBOL', count(*) Successes
from
(
-- trial data
select 
	*
   ,case
		WHEN PYPL_Close_percent_gain > SPY_Close_percent_gain then 'SUCCESS'
		ELSE 'FAIL'
    end SUCCESS_OR_FAIL
from SPYvsPYPL
) for_SUCCESS_COUNT_for_PYPL
group by SUCCESS_OR_FAIL
having SUCCESS_OR_FAIL = 'SUCCESS') for_aggregate_successes
cross join
(select count(*) trials from SPYvsPYPL) for_aggregate_trials
) for_aggregate_trials_with_factorials
)

-- display preliminary results by symbol
select * from Binomial_probability_calculator_for_sign_test


-- calculate and update binomial coeeficient value 
-- and other calculated fields for binomial probability (sPt)
update Binomial_probability_calculator_for_sign_test
set 
	binomial_coeff = t_factorial/(s_factorial*t_less_s_factorial),
    p_to_s_power = power(p,s),
    p_to_t_less_s_power = power(p,t_less_s),
	sPt  = (t_factorial/(s_factorial*t_less_s_factorial)) * power(p,s) * power(p,t_less_s)


-- display final results by symbol
select * from Binomial_probability_calculator_for_sign_test
